#include <Application.h>
#include <stdlib.h>
#include <OS.h>
#include <math.h>
#include <File.h>
#include <Directory.h>
#include <Entry.h>

#include "scarythread.h"
#include "scaryapp.h"
#include "monster.h"
#include "messages.h"

ScaryThread::ScaryThread( BString aUserName, BString aDoorSig )
	: BeBSDoorThread( aUserName, aDoorSig )
{
	thread_id theID;

	theUserName = aUserName;

	// add the scarythread identifier
	aUserName += aDoorSig;

	theID = spawn_thread( Run, aUserName.String(), B_NORMAL_PRIORITY, (void*)this );
	resume_thread( theID );
}

ScaryThread::~ScaryThread()
{
}

void ScaryThread::RunDoor()
{
	isRunning = true;

	// seed the random number generator
	srand( real_time_clock() );
	
	SendString( "\nWelcome to Scary Caverns!" );

//	Setup
	// load the player
	if( (thePlayer.LoadPlayer( theUserName )) == false )
	{
		NewPlayer();
	}
	else
	{
		SendString( "\n\nWelcome back, " );
		SendString( thePlayer.GetName() );
		SendString( "!\n" );
	}

	if( thePlayer.GetHP() <= 0 )
	{
		SendString( "\n  You're currently dead, but if you wait until tomorrow that will probably change." );
		isRunning = false;
	}

	thePlayer.SetLocation( "Town" );

	// This is our main loop
	while( isRunning )
	{
// main loop
		thePlayer.SavePlayer( theUserName );
		// Get a command and process it
		ProcessCommand( Prompt() );

		if( thePlayer.GetHP() <= 0 )
		{
			SendString( "\n You've been killed!  Try again tomorrow!" );
			isRunning = false;
		}
	}

	// Save the player
	thePlayer.SavePlayer( theUserName );
	
// goodbye cruel world!
	CloseDoor();
	exit_thread( 0 );	
}

char ScaryThread::Prompt()
{
	BString tempString;
	char tempChar;

	tempString = "\n<";
	tempString += thePlayer.GetName();
	tempString += " : ";
	tempString += thePlayer.GetLocation();
	if( thePlayer.GetLocation() == "Caves" )
	{
		tempString += " ";
		tempString << thePlayer.GetCaveLevel();
	}
	tempString += "> ";

	SendString( tempString );
	
	tempChar = GetChar();

	// complete command
	if( thePlayer.GetLocation() == "Town" )
	{
		switch( tempChar )
		{
			case 'c': case 'C':
				SendString( "aves" );
				break;
			case 's': case 'S':
				SendString( "tatus" );
				break;
			case 'b': case 'B':
				SendString( "lacksmith" );
				break;
			case 'h': case 'H':
				SendString( "ealer" );
				break;
			case 'w': case 'W':
				SendString( "all of Fame" );
				break;
			case 'q': case 'Q':
				SendString( "uit" );
				break;
			default:
				SendString( "???" );
				break;
		}
	}
	else if( thePlayer.GetLocation() == "Caves" )
	{
		switch( tempChar )
		{
			case 't': case 'T':
				SendString( "own" );
				break;
			case 's': case 'S':
				SendString( "tatus" );
				break;
			case 'f': case 'F':
				SendString( "ind a beastie" );
				break;
			case 'q': case 'Q':
				SendString( "uit" );
				break;
			case 'u': case 'U':
				if( thePlayer.GetCaveLevel() == 1 )
					SendString( "\bYou are on the top level already!" );
				else
					SendString( "p to the next cave level." );
				break;
			case 'd': case 'D':
				if( thePlayer.GetCaveLevel() == 10 )
					SendString( "\bYou're at the bottom already!" );
				else
					SendString( "own to the next cave level." );
				break;
			default:
				SendString( "???" );
				break;
		}
	} else if( thePlayer.GetLocation() == "Blacksmith" )
	{
		switch( tempChar )
		{
			case 'w': case 'W':
				SendString( "\bPurchase Weapon" );
				break;
			case 'a': case 'A':
				SendString( "\bPurchase Armor" );
				break;
			case 's': case 'S':
				SendString( "tatus" );
				break;
			case 't': case 'T':
				SendString( "\bReturn to Town" );
				break;
			case 'q': case 'Q':
				SendString( "uit" );
				break;
			default:
				SendString( "???" );
				break;
		}
	} else if( thePlayer.GetLocation() == "Healer" )
	{
		switch( tempChar )
		{
			case 'h': case 'H':
				SendString( "eal 1 hp." );
				break;
			case 'a': case 'A':
				SendString( "\bHeal All." );
				break;
			case 's': case 'S':
				SendString( "tatus" );
				break;
			case 't': case 'T':
				SendString( "\bReturn to Town" );
				break;
			case 'q': case 'Q':
				SendString( "uit" );
				break;
			default:
				SendString( "???" );
				break;
		}
	}
	
	return tempChar;
}

void ScaryThread::ProcessCommand( char theCommand )
{
	if( thePlayer.GetLocation() == "Town" )
	{
		switch( theCommand )
		{
			case 'c': case 'C':
				thePlayer.SetLocation( "Caves" );
				break;
			case 's': case 'S':
				Status();
				break;
			case 'b': case 'B':
				thePlayer.SetLocation( "Blacksmith" );
				break;
			case 'h': case 'H':
				thePlayer.SetLocation( "Healers" );
				break;
			case 'w': case 'W':
				WallOfFame();
				break;
			case 'q': case 'Q':
				isRunning = false;
				break;
			default:
				Menu();
				break;
		}
	} else if( thePlayer.GetLocation() == "Caves" )
	{
		switch( theCommand )
		{
			case 't': case 'T':
				thePlayer.SetLocation( "Town" );
				break;
			case 's': case 'S':
				Status();
				break;
			case 'f': case 'F':
				Fight();
				break;
			case 'u': case 'U':
				thePlayer.GoUp();
				break;
			case 'd': case 'D':
				thePlayer.GoDown();
				break;
			case 'q': case 'Q':
				isRunning = false;
				break;
			default:
				Menu();
				break;
		}
	} else if( thePlayer.GetLocation() == "Blacksmith" )
	{
		switch( theCommand )
		{
			case 'w': case 'W':
				BuyItem();
				break;
			case 'a': case 'A':
				BuyItem( false );
				break;
			case 't': case 'T':
				thePlayer.SetLocation( "Town" );
				break;
			case 's': case 'S':
				Status();
				break;
			case 'q': case 'Q':
				isRunning = false;
				break;
			default:
				Menu();
				break;
		}
	} else if( thePlayer.GetLocation() == "Healers" )
	{
		switch( theCommand )
		{
			case 'h': case 'H':
				Heal( 1 );
				break;
			case 'a': case 'A':
				Heal( thePlayer.GetMaxHP() - thePlayer.GetHP() );
				break;
			case 's': case 'S':
				Status();
				break;
			case 't': case 'T':
				thePlayer.SetLocation( "Town" );
				break;
			case 'q': case 'Q':
				isRunning = false;
				break;
			default:
				Menu();
				break;
		}
	}
}

void ScaryThread::Menu()
{
	if( thePlayer.GetLocation() == "Town" )
	{
		SendString( "\n  Town command menu:" );
		SendString( "\n (C)aves" );
		SendString( "\n (B)lacksmith" );
		SendString( "\n (H)ealer's Hut" );
		SendString( "\n (S)tatus" );
		SendString( "\n (W)all of Fame" );
		SendString( "\n (Q)uit" );
		SendString( "\n" );
	} else if( thePlayer.GetLocation() == "Caves" )
	{
		SendString( "\n  Caves command menu:" );
		SendString( "\n (F)ind a beastie" );
		if( thePlayer.GetCaveLevel() != 1 )
			SendString( "\n (U)p a cave level" );
		if( thePlayer.GetCaveLevel() != 10 )
			SendString( "\n (D)own a cave level" );
		SendString( "\n (T)own" );
		SendString( "\n (S)tatus" );
		SendString( "\n (Q)uit" );
		SendString( "\n" );
	} else if( thePlayer.GetLocation() == "Blacksmith" )
	{
		SendString( "\n (W)eapons" );
		SendString( "\n (A)rmor" );
		SendString( "\n (S)tatus" );
		SendString( "\n (T)own" );
		SendString( "\n (Q)uit" );
		SendString( "\n" );
	} else if( thePlayer.GetLocation() == "Healers" )
	{
		SendString( "\n (H)eal one" );
		SendString( "\n (A)ll healed" );
		SendString( "\n (S)tatus" );
		SendString( "\n (T)own" );
		SendString( "\n (Q)uit" );
		SendString( "\n\nCurrent Pricing: " );
		SendNumber( ( thePlayer.GetLevel() / 4 ) );
		SendString( "/HP." );
		SendString( "\n" );
	}
}

void ScaryThread::NewPlayer()
{
	BString tempString;
	char tempChar;
	bool isChoosing;

	SendString( "\n\nIt looks like you're new here.\n");

	isChoosing = true;
	while( isChoosing )
	{
		SendString( "\nWhat name would you like? [" );
		SendString( theUserName );
		SendString( "] : " );
	
		tempString = GetString();
	
		if( tempString == "" )
		{
			tempString = theUserName;
		}
	
		SendString( "\n[" );
		SendString( tempString.String() );
		SendString( "] is this correct?" );
		tempChar = GetChar();
	
		if( tempChar == 'y' || tempChar == 'Y' )
		{
			isChoosing = false;
		}
	}
	
	thePlayer.SetName( tempString );
	
	// initial gold alotment
	thePlayer.AddGold( 100 );
}

void ScaryThread::Status()
{
	BString tempString;

	SendString( "\nStatus for [" );
	SendString( thePlayer.GetName() );
	SendString( "]:" );

	SendString( "\n Level: " );
	SendNumber( thePlayer.GetLevel() );

	SendString( "\n HP: " );
	SendNumber( thePlayer.GetHP() );
	SendString( "/" );
	SendNumber( thePlayer.GetMaxHP() );

	SendString( "\n XP: " );
	SendNumber( thePlayer.GetXP() );
	SendString( "/" );
	SendNumber( pow( 2, (thePlayer.GetLevel() + 4) ) );
	
	SendString( "\n Str:" );
	SendNumber( thePlayer.GetStr() );
	SendString( " + " );
	SendNumber( pow( 2, thePlayer.GetWeaponStr() ) );
	SendString( " Def: " );
	SendNumber( thePlayer.GetDef() );
	SendString( " + " );
	SendNumber( pow( 2, thePlayer.GetArmorStr() ) );

	SendString( "\n Weapon: " );
	SendString( thePlayer.GetWeapon() );

	SendString( "\n Armor: " );
	SendString( thePlayer.GetArmor() );

	SendString( "\n Gold: " );
	SendNumber( thePlayer.GetBalance() );
	
	SendString( "\n Cave Fights left for today: " );
	SendNumber( thePlayer.GetFights() );
	
	SendString( "\n" );
}

void ScaryThread::Heal( int healed )
{
	BString tempString;

	if( thePlayer.GetHP() == thePlayer.GetMaxHP() )
	{
		SendString( "\n You are not in need of healing currently." );
		return;
	}
	
	if( thePlayer.SpendGold( ( thePlayer.GetLevel() / 5 ) * healed ) == true )
	{
		thePlayer.Heal( healed );
		SendString( "\nHealed " );
		SendNumber( healed );
		SendString( " HP for " );
		SendNumber( ( thePlayer.GetLevel() / 5 ) * healed );
		SendString( " gold." );
	}
	else
	{
		SendString( "\nYou don't appear to have " );
		SendNumber( ( thePlayer.GetLevel() / 5 ) * healed );
		SendString( " gold right now." );
	}
}

void ScaryThread::SendNumber( int number )
{
	BString tempString( "" );
	tempString << number;
	SendString( tempString );
}

void ScaryThread::Fight( bool isBigBoss = false )
{
	CMonster theMonster;
	bool isFighting;
	char tempChar;
	int plAtk = 0;
	int moAtk = 0;
	BString tempString;
	
	if( thePlayer.Fights() == false )
	{
		SendString( "\n You are far too tired to do this all day, you know." );
		return;
	}

	if( isBigBoss == false )
		theMonster.GetMonster( thePlayer.GetCaveLevel() );
	else
		theMonster.GetMonster( -1 );

	if( theMonster.GetName() == "***SPECIAL EVENT***" )
	{
		SpecialEvent();
		return;
	}	

	SendString( "\nYou are fighting a viscious looking " );
	SendString( theMonster.GetName() );
	SendString( "." );

	// actually fight
	isFighting = true;
	
	while( isFighting )
	{
		SendString( "\n\n{ " );
		SendString( theMonster.GetName() );
		SendString( " } " );
		SendNumber( theMonster.GetHP() );
		SendString( " HP" );
		SendString( "\n[" );
		SendString( thePlayer.GetName() );
		SendString( "] " );
		SendNumber( thePlayer.GetHP() );
		SendString( "/" );
		SendNumber( thePlayer.GetMaxHP() );
		SendString( "\n\nWhat will you do?" );
		
		tempChar = GetChar();
		
		switch( tempChar )
		{
			case 'a': case 'A':
				SendString( "Attack!" );
				break;
			case 'r': case 'R':
				if( isBigBoss == false )
				{
					SendString( "Run!" );
					return;
				}
				else
				{
					SendString( "You try to run, but the gigantic beastie is just too fast!" );
				}
				break;
			default:
				SendString( "???\n (A)ttack\n (R)un" );
				break;
		}

		plAtk = 0;
		moAtk = 0;

		if( tempChar == 'a' || tempChar == 'A' )
		{
			while( plAtk == 0 && moAtk == 0 )
			{
				plAtk = theMonster.IsHitBy( thePlayer.GetStr() + pow( 2, thePlayer.GetWeaponStr() ));
				moAtk = thePlayer.IsHitBy( theMonster.GetStr() );
			}
		}

		if( tempChar == 'a' || tempChar == 'A' )
		{
			if( plAtk == 0 )
			{
				SendString( "\n\nYou missed the " );
				SendString( theMonster.GetName() );
				SendString( ".  Don't you look foolish." );
			}
			else
			{
				// crit hit or normal/show status of hit.
				if( plAtk < (.75 * (thePlayer.GetStr() + pow( 2, thePlayer.GetWeaponStr() ) ) ) )
				{
					SendString( "\n\nYou hit the " );
					SendString( theMonster.GetName() );
					SendString( " with your " );
					SendString( thePlayer.GetWeapon() );
					SendString( " for " );
					SendNumber( plAtk );
					SendString( " HP." );
				}
				else
				{
					tempString = "\n\n***CRITICAL HIT!!!***\n You strike the ";
					tempString += theMonster.GetName();
					tempString += " with skill so great you startle even yourself.\n\n  ";
					tempString << plAtk;
					tempString += " hp are sprayed across the ground, splattered by your mighty ";
					tempString += thePlayer.GetWeapon();
					tempString += ".";
					SendString( tempString );
				}

				if( theMonster.GetHP() <= 0 )
				{
					thePlayer.Heal( moAtk );	// heal anything the monster did.
				
					SendString( "\n  You killed the " );
					SendString( theMonster.GetName() );
					SendString( "!\n Got " );
					SendNumber( theMonster.GetXP() );
					SendString( " XP and " );
					SendNumber( theMonster.GetGold() );
					SendString( " gold." );

					thePlayer.AddGold( theMonster.GetGold() );
					if( thePlayer.AddXP( theMonster.GetXP() ) == true )
					{
						SendString( "\n You gained a level!\n" );
						Status();
					}
					
					if( isBigBoss == true )
					{
						WinGame();
					}
					return;
				}
			}

			if( moAtk == 0 )
			{
				SendString( "\n" );
				SendString( theMonster.GetName() );
				SendString( " stumbles around flailing miserably." );
			}
			else
			{
				SendString( "\n" );
				SendString( theMonster.GetName() );
				SendString( " strikes with its " );
				SendString( theMonster.GetAttack() );
				SendString( " for " );
				SendNumber( moAtk );
				SendString( " HP." );
			}
		}

		if( thePlayer.GetHP() <= 0 )
			return;
	}
}

void ScaryThread::BuyItem( bool isWeapon = true )
{
	BFile* theFile;
	BString tempString;
	char tempChar;
	int numOfItems;
	int selection;
	int i;
	BList theItemList;
	int startPoint;
	
	if( isWeapon )
		theFile = new BFile( "/boot/home/BeBS/doors/scary/weapons", B_READ_ONLY );
	else
		theFile = new BFile( "/boot/home/BeBS/doors/scary/armor", B_READ_ONLY );
	
	tempString = "";
	tempChar = '\0';
	
	while( tempChar != '\n' )
	{
		theFile->Read( (void*)&tempChar, sizeof( char ) );
		tempString += tempChar;
	}
	numOfItems = atol( tempString.String() );

	if( isWeapon )
	{
		if( numOfItems > ( thePlayer.GetWeaponStr() + 4 ) )
			numOfItems = thePlayer.GetWeaponStr() + 4;
		startPoint = numOfItems - 10;
		if( startPoint < 0 )
			startPoint = 0;
	}
	else
	{
		if( numOfItems > ( thePlayer.GetArmorStr() + 4 ) )
			numOfItems = thePlayer.GetArmorStr() + 4;
		startPoint = numOfItems - 10;
		if( startPoint < 0 )
			startPoint = 0;
	}
	
	for( i = 0; i < numOfItems; i++ )
	{
		tempChar = '\0';
		tempString = "";
		while( tempChar != '\n' )
		{
			theFile->Read( (void*)&tempChar, sizeof( char ) );
			if( tempChar != '\n' )
				tempString += tempChar;
		}
		theItemList.AddItem( (void*)(new BString( tempString ) ) );
		
		if( i >= startPoint )
		{
			SendString( "\n(" );
			SendNumber( i + 1 );
			SendString( ") " );

			SendString( tempString );
			SendString( " : " );
			if( isWeapon )
			{
				SendNumber( 50 * pow( 2, i ) );
				if( i == ( thePlayer.GetWeaponStr() - 1 ) )
					SendString( " *** " );
			}
			else
			{
				SendNumber( 40 * pow( 2, i ) );
				if( i == ( thePlayer.GetArmorStr() - 1 ) )
					SendString( " *** " );
			}
		}
	}
	
	SendString( "\n\nWhich would you like to buy?" );
	tempString = GetString();
	
	selection = 0;
	selection = atol( tempString.String() );
	if( selection < 1 || selection > numOfItems )
	{
		SendString( "???" );
		return;
	}

	if( isWeapon )
	{
		if( thePlayer.GetWeaponStr() >= selection )
		{
			SendString( "\nYou don't really want that one.  You've got a better one!" );
			return;
		}
	
		if( thePlayer.SpendGold( 25 * pow( 2, selection ) ) == false )
		{
			SendString( "\nYou don't have that kind of cash!" );
			return;
		}
	}
	else
	{
		if( thePlayer.GetArmorStr() >= selection )
		{
			SendString( "\nYou don't really want that one.  You've got a better one!" );
			return;
		}
	
		if( thePlayer.SpendGold( 20 * pow( 2, selection ) ) == false )
		{
			SendString( "\nYou don't have that kind of cash!" );
			return;
		}
	}
	
	// see if that's what the player REALLY wants
	SendString( "\nAre you sure you want to buy that " );
	tempString = *((BString*)(theItemList.ItemAt( selection - 1 )));
	SendString( tempString );
	SendString( "? (y/n) " );

	tempChar = GetChar();
	if( tempChar == 'y' || tempChar == 'Y' )
	{
		// actually set the weapon
		if( isWeapon )
			thePlayer.SetWeapon( tempString, selection );
		else
			thePlayer.SetArmor( tempString, selection );
		
		SendString( "\nThanks for buying that old " );
		SendString( tempString );
		SendString( ".  I didn't want it anyway." );
	}
	else
	{
		SendString( "\nThen why did you ask for it in the first place?" );
		if( isWeapon )
			thePlayer.AddGold( 25 * pow( 2, selection ) );
		else
			thePlayer.AddGold( 20 * pow( 2, selection ) );
	}

		
	theFile->Unset();
	delete theFile;
	
	while( theItemList.ItemAt( 0 ) != NULL )
		delete (BString*)(theItemList.RemoveItem( (int32)0 ) );
}

void ScaryThread::WallOfFame()
{
	BDirectory theDirectory( "/boot/home/BeBS/doors/scary/users/" );
	BEntry theEntry;
	BString tempString( "" );
	char buffer[9999] = { '\0' };
	CPlayer tempPlayer;

	SendString( "\n  Welcome to The Wall of Fame\nName : level" );
	
	theDirectory.Rewind();
	
	if( theDirectory.InitCheck() != B_OK )
	{
		SendString( "The users list failed to initialize." );
	}
	
	while( theDirectory.GetNextEntry( &theEntry ) == B_OK )
	{
		theEntry.GetName( buffer );
		tempString = "";
		tempString = buffer;

		tempPlayer.LoadPlayer( tempString );
		SendString( "\n" );
		SendString( tempPlayer.GetName() );
		SendString( " : " );
		SendNumber( tempPlayer.GetLevel() );
	}
	
	SendString( "\n" );
	theDirectory.Unset();
}

void ScaryThread::SpecialEvent()
{
	int randNum;
	int i;
	BString tempString;
	
	thePlayer.AddFights( 1 );

	SendString( "\n\n***Special Event!***\n\n" );

	if( thePlayer.GetCaveLevel() == 10 )
	{
		randNum = rand() % 10;
		if( randNum == 1 )
		{
			Fight( true );
			return;
		}
	}

	randNum = rand() % 5;
	
	switch( randNum )
	{
		case 0:
			SendString( "You find a sparkling pool deep in the caverns.  Its subtle beauty refreshes you." );
			SendString( "\n\nYou heal " );
			SendNumber( thePlayer.Heal( 99999 ) );
			SendString( " HP." );
			break;
		case 1:
			tempString = "You find a luscious-looking cave-apple tree.  After eating all ";
			tempString += "the apples on the tree, you feel like you could take on the world!";
			tempString += "\n\nYou gain 3 cave fights!";
			SendString( tempString );
			thePlayer.AddFights( 3 );
			break;
		case 2:
			tempString = "You happen upon a lonely monk who offers to train you in the fighting ";
			tempString += "arts.  After several hours of moving rocks and standing on your head ";
			tempString += "you feel more than just a little silly.  When you get done with your ";
			tempString += "last exercise of covering your eyes, you find that the monk has left.";
			tempString += "\n\nYou gain ";
			i = rand() % thePlayer.GetLevel();
			i++;
			tempString << i;
			tempString += " Strength!";
			thePlayer.AddStr( i );
			SendString( tempString );
			break;
		case 3:
			tempString = "You find the rotting corpse of a not-as-lucky-as-you-are warrior.  ";
			tempString += "Pilfering through its pockets, you find ";
			i = rand() % (thePlayer.GetLevel() * 10);
			i++;
			tempString << i;
			tempString += " gold that the others somehow missed!";
			thePlayer.AddGold( i );
			SendString( tempString );
			break;
		case 4:
			tempString = "As you stumble blindly about in the dark caves, you accidentally walk ";
			tempString += "through a giant wild bramble patch.  When you finally make your way ";
			tempString += "out of the thicket, you turn to discover that the patch has mysteriously ";
			tempString += "vanished.  Your scratches have healed, hardening your skin.";
			tempString += "\n\nYou gain ";
			i = rand() % thePlayer.GetLevel();
			i++;
			tempString << i;
			tempString += " Defense!";
			thePlayer.AddDef( i );
			SendString( tempString );
			break;
		default:
			break;
	}
	
	SendString( "\n" );
}

void ScaryThread::WinGame()
{
	CPlayer newPlayer;
	BString tempString;
	
	tempString = "*** YOU WIN!!! ***\n\n";
	tempString += "  You walk out of the caves, victorious over the nastiest of the beasties in the ";
	tempString += "caves.  Stories of your glorious victory over the beast soon spread among the ";
	tempString += "townsfolk.  But, like all great battle tales, the stories soon become embellished ";
	tempString += "and grow into Legend.\n";
	tempString += "  More rapidly than you might like, the stories and your experiences fade into ";
	tempString += "obscurity, until another hero is born to slay the greatest of the beasties...";
	tempString += "\n\n  Thank you for playing Scary Caverns!  Your character has now been reset.";
	SendString( tempString );
	
	newPlayer.SetName( thePlayer.GetName() );
	newPlayer.AddGold( 100 );
	newPlayer.SetLocation( "Town" );
	newPlayer.SavePlayer( theUserName );
	thePlayer.LoadPlayer( theUserName );
}